==> mule-module-extensions-support-4.11.0-20251030.173301-137-sources.jar: org/mule/runtime/module/extension/internal/loader/delegate/ParameterModelsLoaderDelegate.java <==
/*
 * Copyright 2023 Salesforce, Inc. All rights reserved.
 * The software in this package is published under the terms of the CPAL v1.0
 * license, a copy of which has been included with this distribution in the
 * LICENSE.txt file.
 */
package org.mule.runtime.module.extension.internal.loader.delegate;

import static org.mule.runtime.api.meta.ExpressionSupport.NOT_SUPPORTED;
import static org.mule.runtime.api.meta.model.parameter.ParameterGroupModel.DEFAULT_GROUP_NAME;
import static org.mule.runtime.module.extension.internal.loader.parser.java.utils.MinMuleVersionUtils.declarerWithMmv;
import static org.mule.runtime.module.extension.internal.loader.utils.FieldValueProviderNameUtils.getFieldValueProviderName;
import static org.mule.runtime.module.extension.internal.loader.utils.ModelLoaderUtils.addSemanticTerms;

import static java.util.stream.Collectors.toMap;

import org.mule.metadata.api.model.MetadataType;
import org.mule.metadata.api.model.StringType;
import org.mule.runtime.api.meta.model.ModelProperty;
import org.mule.runtime.api.meta.model.declaration.fluent.HasParametersDeclarer;
import org.mule.runtime.api.meta.model.declaration.fluent.ParameterDeclarer;
import org.mule.runtime.api.meta.model.declaration.fluent.ParameterGroupDeclarer;
import org.mule.runtime.api.meta.model.parameter.ActingParameterModel;
import org.mule.runtime.api.meta.model.parameter.FieldValueProviderModel;
import org.mule.runtime.api.meta.model.parameter.ValueProviderModel;
import org.mule.runtime.api.util.LazyValue;
import org.mule.runtime.api.util.Pair;
import org.mule.runtime.extension.api.loader.ExtensionLoadingContext;
import org.mule.runtime.extension.api.loader.parser.ActingParameterModelParser;
import org.mule.runtime.extension.api.loader.parser.FieldValueProviderModelParser;
import org.mule.runtime.extension.api.loader.parser.ParameterGroupModelParser;
import org.mule.runtime.extension.api.loader.parser.ParameterModelParser;
import org.mule.runtime.extension.api.loader.parser.ValueProviderModelParser;
import org.mule.runtime.extension.api.loader.parser.metadata.InputResolverModelParser;
import org.mule.runtime.extension.api.model.parameter.ImmutableActingParameterModel;
import org.mule.runtime.extension.api.property.ExcludeFromConnectivitySchemaModelProperty;
import org.mule.runtime.extension.api.property.MetadataKeyPartModelProperty;
import org.mule.runtime.module.extension.internal.loader.java.property.FieldsValueProviderFactoryModelProperty;
import org.mule.runtime.module.extension.internal.loader.java.property.ValueProviderFactoryModelProperty;
import org.mule.runtime.module.extension.internal.loader.parser.java.utils.ResolvedMinMuleVersion;
import org.mule.runtime.module.extension.internal.util.IntrospectionUtils;

import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.function.Consumer;
import java.util.function.Supplier;

public final class ParameterModelsLoaderDelegate {

  private final Supplier<StereotypeModelLoaderDelegate> stereotypeModelLoader;
  private final Consumer<MetadataType> typeRegisterer;

  public ParameterModelsLoaderDelegate(Supplier<StereotypeModelLoaderDelegate> stereotypeModelLoader,
                                       Consumer<MetadataType> typeRegisterer) {
    this.stereotypeModelLoader = stereotypeModelLoader;
    this.typeRegisterer = typeRegisterer;
  }

  public List<ParameterDeclarer<?>> declare(HasParametersDeclarer component,
                                            List<ParameterGroupModelParser> groupParsers,
                                            ExtensionLoadingContext context) {
    final List<ParameterDeclarer<?>> declarerList = new LinkedList<>();

    // Map from unaliased name to aliased name.
    // Because this will be used only for ValueProviders we can afford to flatten by parameter name.
    // ValueProviders are not supported on components that have repeated names in different groups.
    final Supplier<Map<String, String>> containerParameterNames = new LazyValue<>(() -> getContainerParameterNames(groupParsers));

    groupParsers.forEach(group -> {
      ParameterGroupDeclarer<?> groupDeclarer;

      if (DEFAULT_GROUP_NAME.equals(group.getName())) {
        groupDeclarer = component.onDefaultParameterGroup();
      } else {
        groupDeclarer = component.onParameterGroup(group.getName())
            .withDslInlineRepresentation(group.showsInDsl());

        group.getDisplayModel().ifPresent(groupDeclarer::withDisplayModel);
        group.getLayoutModel().ifPresent(groupDeclarer::withLayout);
        group.getAdditionalModelProperties().forEach(groupDeclarer::withModelProperty);
        groupDeclarer.getDeclaration().setDescription(group.getDescription());
        group.getLayoutModel().ifPresent(groupDeclarer::withLayout);
      }
      group.getExclusiveOptionals().ifPresent(descriptor -> groupDeclarer
          .withExclusiveOptionals(descriptor.getExclusiveOptionals(), descriptor.isOneRequired()));

      group.getParameterParsers().forEach(parameterParser -> {
        ParameterDeclarer<?> parameter;
        if (parameterParser.isRequired()) {
          parameter = groupDeclarer.withRequiredParameter(parameterParser.getName());
        } else {
          parameter = groupDeclarer.withOptionalParameter(parameterParser.getName())
              .defaultingTo(parameterParser.getDefaultValue());
        }

        parameterParser.getMetadataKeyPart()
            .map(this::createMetadataKeyPartModelProperty)
            .ifPresent(parameter::withModelProperty);

        Optional<InputResolverModelParser> inputResolverModelParser = parameterParser.getInputResolverModelParser();
        final MetadataType metadataType = parameterParser.getType();
        if (inputResolverModelParser.isPresent()) {
          parameter.ofDynamicType(metadataType);
        } else {
          parameter.ofType(metadataType);
        }
        parameter.describedAs(parameterParser.getDescription())
            .withRole(parameterParser.getRole())
            .withExpressionSupport(parameterParser.getExpressionSupport());

        typeRegisterer.accept(metadataType);

        if (parameterParser.isComponentId()) {
          parameter.asComponentId();
        }

        if (parameterParser.isConfigOverride()) {
          parameter.asConfigOverride();
        }

        declareValueProviders(group, parameterParser, parameter, containerParameterNames);
        declareFieldValueProviders(parameterParser, parameter, containerParameterNames);

        if (parameterParser.isExcludedFromConnectivitySchema()) {
          parameter.withModelProperty(new ExcludeFromConnectivitySchemaModelProperty());
        }

        parameterParser.getLayoutModel().ifPresent(parameter::withLayout);
        parameterParser.getDslConfiguration().ifPresent(parameter::withDsl);
        parameterParser.getDeprecationModel().ifPresent(parameter::withDeprecation);
        parameterParser.getDisplayModel().ifPresent(parameter::withDisplayModel);
        parameterParser.getOAuthParameterModelProperty().ifPresent(parameter::withModelProperty);
        parameterParser.getAdditionalModelProperties().forEach(parameter::withModelProperty);
        if (context.isResolveMinMuleVersion()) {
          parameterParser.getResolvedMinMuleVersion()
              .ifPresent(mmv -> declarerWithMmv(parameter, new ResolvedMinMuleVersion(parameterParser.getName(),
                                                                                      mmv.getMinMuleVersion(), mmv.getReason())));
        }

        addSemanticTerms(parameter.getDeclaration(), parameterParser);
        stereotypeModelLoader.get().addStereotypes(parameterParser, parameter);
        declarerList.add(parameter);
      });

      if (declarerList.stream().noneMatch(p -> p.getDeclaration().isComponentId())) {
        declarerList.stream()
            .filter(p -> p.getDeclaration().getName().equals("name")
                && p.getDeclaration().isRequired()
                && p.getDeclaration().getExpressionSupport() == NOT_SUPPORTED
                && p.getDeclaration().getType() instanceof StringType
                && p.getDeclaration().getAllowedStereotypeModels().isEmpty())
            .forEach(ParameterDeclarer::asComponentId);
      }
    });

    return declarerList;
  }

  // Logic was extracted from what used to be the ValueProvidersParameterDeclarationEnricher in earlier versions.
  private void declareValueProviders(ParameterGroupModelParser group, ParameterModelParser parameterParser,
                                     ParameterDeclarer<?> parameter, Supplier<Map<String, String>> containerParameterNames) {
    parameterParser.getValueProviderModelParser()
        .ifPresent(valueProviderModelParser -> {
          List<ActingParameterModel> actingParameterModels =
              getActingParameterModels(valueProviderModelParser, containerParameterNames.get());
          String vpName = valueProviderModelParser.getPartOrder().isPresent() ? group.getName() : parameterParser.getName();
          parameter.withValueProviderModel(createValueProviderModelFromParser(vpName,
                                                                              valueProviderModelParser,
                                                                              actingParameterModels));
          parameter
              .withModelProperty(new ValueProviderFactoryModelProperty(valueProviderModelParser.getValueProviderFactory(),
                                                                       actingParameterModels));
        });
  }

  private void declareFieldValueProviders(ParameterModelParser parameterParser,
                                          ParameterDeclarer<?> parameter, Supplier<Map<String, String>> containerParameterNames) {
    Map<String, ValueProviderFactoryModelProperty> fieldValueProviderFactoriesBySelector = new LinkedHashMap<>();
    List<FieldValueProviderModel> fieldValueProviderModels = new ArrayList<>();
    parameterParser.getFieldValueProvidersModelParser()
        .forEach(fieldValuesParser -> {
          List<ActingParameterModel> actingParameterModels =
              getActingParameterModels(fieldValuesParser, containerParameterNames.get());
          fieldValueProviderModels.addAll(createValueProviderModelsFromParser(parameterParser.getName(),
                                                                              fieldValuesParser,
                                                                              actingParameterModels));
          ValueProviderFactoryModelProperty factoryMP =
              new ValueProviderFactoryModelProperty(fieldValuesParser.getValueProviderFactory(), actingParameterModels);
          fieldValuesParser.getTargetSelectors()
              .forEach(targetSelector -> fieldValueProviderFactoriesBySelector.put(targetSelector, factoryMP));
        });

    if (!fieldValueProviderModels.isEmpty()) {
      parameter.withFieldValueProviderModels(fieldValueProviderModels);
    }
    if (!fieldValueProviderFactoriesBySelector.isEmpty()) {
      parameter.withModelProperty(new FieldsValueProviderFactoryModelProperty(fieldValueProviderFactoriesBySelector));
    }
  }

  private Map<String, String> getContainerParameterNames(List<ParameterGroupModelParser> groupParsers) {
    return groupParsers.stream()
        .flatMap(gp -> gp.getParameterParsers().stream())
        .collect(toMap(IntrospectionUtils::getImplementingName, ParameterModelParser::getName));
  }

  private ValueProviderModel createValueProviderModelFromParser(String valueProviderName,
                                                                ValueProviderModelParser valueProviderModelParser,
                                                                List<ActingParameterModel> actingParameterModels) {
    return new ValueProviderModel(actingParameterModels,
                                  valueProviderModelParser.requiresConfiguration(),
                                  valueProviderModelParser.requiresConnection(),
                                  valueProviderModelParser.isOpen(),
                                  valueProviderModelParser.getPartOrder().orElse(1),
                                  valueProviderName,
                                  valueProviderModelParser.getId(),
                                  valueProviderModelParser.getAdditionalModelProperties().toArray(new ModelProperty[0]));
  }

  private List<FieldValueProviderModel> createValueProviderModelsFromParser(String valueProviderName,
                                                                            FieldValueProviderModelParser valueProviderModelParser,
                                                                            List<ActingParameterModel> actingParameterModels) {
    List<FieldValueProviderModel> fieldValueProviderModels = new ArrayList<>();
    int partOrder = 1;
    valueProviderName = getFieldValueProviderName(valueProviderName,
                                                  valueProviderModelParser.getTargetSelectors().toArray(new String[0]));
    for (String targetSelector : valueProviderModelParser.getTargetSelectors()) {
      fieldValueProviderModels.add(new FieldValueProviderModel(actingParameterModels,
                                                               valueProviderModelParser.requiresConfiguration(),
                                                               valueProviderModelParser.requiresConnection(),
                                                               valueProviderModelParser.isOpen(),
                                                               partOrder++,
                                                               valueProviderName,
                                                               valueProviderModelParser.getId(),
                                                               targetSelector));

    }

    return fieldValueProviderModels;
  }

  private List<ActingParameterModel> getActingParameterModels(ValueProviderModelParser valueProviderModelParser,
                                                              Map<String, String> containerParameterNames) {
    return valueProviderModelParser.getActingParameterParsers().stream()
        .map(parameterParser -> getActingParameterModel(parameterParser, containerParameterNames))
        .toList();
  }

  private ActingParameterModel getActingParameterModel(ActingParameterModelParser actingParameterModelParser,
                                                       Map<String, String> containerParameterNames) {
    if (actingParameterModelParser.getExtractionExpression().isPresent()) {
      return new ImmutableActingParameterModel(actingParameterModelParser.getParameterName(),
                                               actingParameterModelParser.isRequired(),
                                               actingParameterModelParser.getExtractionExpression().get());

    } else {
      String aliasedName = containerParameterNames.get(actingParameterModelParser.getParameterName());
      return new ImmutableActingParameterModel(aliasedName, actingParameterModelParser.isRequired(), aliasedName);
    }
  }

  private MetadataKeyPartModelProperty createMetadataKeyPartModelProperty(Pair<Integer, Boolean> metadataKeyPart) {
    return new MetadataKeyPartModelProperty(metadataKeyPart.getFirst(), metadataKeyPart.getSecond());
  }
}

==> mule-module-extensions-support-4.11.0-20251128.222817-176-sources.jar: org/mule/runtime/module/extension/internal/loader/delegate/ParameterModelsLoaderDelegate.java <==
/*
 * Copyright 2023 Salesforce, Inc. All rights reserved.
 * The software in this package is published under the terms of the CPAL v1.0
 * license, a copy of which has been included with this distribution in the
 * LICENSE.txt file.
 */
package org.mule.runtime.module.extension.internal.loader.delegate;

import static org.mule.runtime.api.meta.ExpressionSupport.NOT_SUPPORTED;
import static org.mule.runtime.api.meta.model.parameter.ParameterGroupModel.DEFAULT_GROUP_NAME;
import static org.mule.runtime.module.extension.internal.loader.parser.java.utils.MinMuleVersionUtils.declareWithBaselineMmv;
import static org.mule.runtime.module.extension.internal.loader.utils.FieldValueProviderNameUtils.getFieldValueProviderName;
import static org.mule.runtime.module.extension.internal.loader.utils.ModelLoaderUtils.addSemanticTerms;

import static java.util.stream.Collectors.toMap;

import org.mule.metadata.api.model.MetadataType;
import org.mule.metadata.api.model.StringType;
import org.mule.runtime.api.meta.model.ModelProperty;
import org.mule.runtime.api.meta.model.declaration.fluent.ExtensionDeclarer;
import org.mule.runtime.api.meta.model.declaration.fluent.HasParametersDeclarer;
import org.mule.runtime.api.meta.model.declaration.fluent.ParameterDeclarer;
import org.mule.runtime.api.meta.model.declaration.fluent.ParameterGroupDeclarer;
import org.mule.runtime.api.meta.model.parameter.ActingParameterModel;
import org.mule.runtime.api.meta.model.parameter.FieldValueProviderModel;
import org.mule.runtime.api.meta.model.parameter.ValueProviderModel;
import org.mule.runtime.api.util.LazyValue;
import org.mule.runtime.api.util.Pair;
import org.mule.runtime.extension.api.loader.ExtensionLoadingContext;
import org.mule.runtime.extension.api.loader.parser.ActingParameterModelParser;
import org.mule.runtime.extension.api.loader.parser.FieldValueProviderModelParser;
import org.mule.runtime.extension.api.loader.parser.ParameterGroupModelParser;
import org.mule.runtime.extension.api.loader.parser.ParameterModelParser;
import org.mule.runtime.extension.api.loader.parser.ValueProviderModelParser;
import org.mule.runtime.extension.api.loader.parser.metadata.InputResolverModelParser;
import org.mule.runtime.extension.api.model.parameter.ImmutableActingParameterModel;
import org.mule.runtime.extension.api.property.ExcludeFromConnectivitySchemaModelProperty;
import org.mule.runtime.extension.api.property.MetadataKeyPartModelProperty;
import org.mule.runtime.module.extension.internal.loader.java.property.FieldsValueProviderFactoryModelProperty;
import org.mule.runtime.module.extension.internal.loader.java.property.ValueProviderFactoryModelProperty;
import org.mule.runtime.module.extension.internal.util.IntrospectionUtils;

import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.function.Consumer;
import java.util.function.Supplier;

public final class ParameterModelsLoaderDelegate {

  private final Supplier<StereotypeModelLoaderDelegate> stereotypeModelLoader;
  private final Consumer<MetadataType> typeRegisterer;

  public ParameterModelsLoaderDelegate(Supplier<StereotypeModelLoaderDelegate> stereotypeModelLoader,
                                       Consumer<MetadataType> typeRegisterer) {
    this.stereotypeModelLoader = stereotypeModelLoader;
    this.typeRegisterer = typeRegisterer;
  }

  public List<ParameterDeclarer<?>> declare(HasParametersDeclarer component,
                                            List<ParameterGroupModelParser> groupParsers,
                                            ExtensionDeclarer extensionDeclarer,
                                            ExtensionLoadingContext context) {
    final List<ParameterDeclarer<?>> declarerList = new LinkedList<>();

    // Map from unaliased name to aliased name.
    // Because this will be used only for ValueProviders we can afford to flatten by parameter name.
    // ValueProviders are not supported on components that have repeated names in different groups.
    final Supplier<Map<String, String>> containerParameterNames = new LazyValue<>(() -> getContainerParameterNames(groupParsers));

    groupParsers.forEach(group -> {
      ParameterGroupDeclarer<?> groupDeclarer;

      if (DEFAULT_GROUP_NAME.equals(group.getName())) {
        groupDeclarer = component.onDefaultParameterGroup();
      } else {
        groupDeclarer = component.onParameterGroup(group.getName())
            .withDslInlineRepresentation(group.showsInDsl());

        group.getDisplayModel().ifPresent(groupDeclarer::withDisplayModel);
        group.getLayoutModel().ifPresent(groupDeclarer::withLayout);
        group.getAdditionalModelProperties().forEach(groupDeclarer::withModelProperty);
        groupDeclarer.getDeclaration().setDescription(group.getDescription());
        group.getLayoutModel().ifPresent(groupDeclarer::withLayout);
      }
      group.getExclusiveOptionals().ifPresent(descriptor -> groupDeclarer
          .withExclusiveOptionals(descriptor.getExclusiveOptionals(), descriptor.isOneRequired()));

      group.getParameterParsers().forEach(parameterParser -> {
        ParameterDeclarer<?> parameter;
        if (parameterParser.isRequired()) {
          parameter = groupDeclarer.withRequiredParameter(parameterParser.getName());
        } else {
          parameter = groupDeclarer.withOptionalParameter(parameterParser.getName())
              .defaultingTo(parameterParser.getDefaultValue());
        }

        parameterParser.getMetadataKeyPart()
            .map(this::createMetadataKeyPartModelProperty)
            .ifPresent(parameter::withModelProperty);

        Optional<InputResolverModelParser> inputResolverModelParser = parameterParser.getInputResolverModelParser();
        final MetadataType metadataType = parameterParser.getType();
        if (inputResolverModelParser.isPresent()) {
          parameter.ofDynamicType(metadataType);
        } else {
          parameter.ofType(metadataType);
        }
        parameter.describedAs(parameterParser.getDescription())
            .withRole(parameterParser.getRole())
            .withExpressionSupport(parameterParser.getExpressionSupport());

        typeRegisterer.accept(metadataType);

        if (parameterParser.isComponentId()) {
          parameter.asComponentId();
        }

        if (parameterParser.isConfigOverride()) {
          parameter.asConfigOverride();
        }

        declareValueProviders(group, parameterParser, parameter, containerParameterNames);
        declareFieldValueProviders(parameterParser, parameter, containerParameterNames);

        if (parameterParser.isExcludedFromConnectivitySchema()) {
          parameter.withModelProperty(new ExcludeFromConnectivitySchemaModelProperty());
        }

        parameterParser.getLayoutModel().ifPresent(parameter::withLayout);
        parameterParser.getDslConfiguration().ifPresent(parameter::withDsl);
        parameterParser.getDeprecationModel().ifPresent(parameter::withDeprecation);
        parameterParser.getDisplayModel().ifPresent(parameter::withDisplayModel);
        parameterParser.getOAuthParameterModelProperty().ifPresent(parameter::withModelProperty);
        parameterParser.getAdditionalModelProperties().forEach(parameter::withModelProperty);
        declareWithBaselineMmv(parameterParser::getResolvedMinMuleVersion, parameter,
                               extensionDeclarer.getDeclaration().getMinMuleVersion(), context);

        addSemanticTerms(parameter.getDeclaration(), parameterParser);
        stereotypeModelLoader.get().addStereotypes(parameterParser, parameter);
        declarerList.add(parameter);
      });

      if (declarerList.stream().noneMatch(p -> p.getDeclaration().isComponentId())) {
        declarerList.stream()
            .filter(p -> p.getDeclaration().getName().equals("name")
                && p.getDeclaration().isRequired()
                && p.getDeclaration().getExpressionSupport() == NOT_SUPPORTED
                && p.getDeclaration().getType() instanceof StringType
                && p.getDeclaration().getAllowedStereotypeModels().isEmpty())
            .forEach(ParameterDeclarer::asComponentId);
      }
    });

    return declarerList;
  }

  // Logic was extracted from what used to be the ValueProvidersParameterDeclarationEnricher in earlier versions.
  private void declareValueProviders(ParameterGroupModelParser group, ParameterModelParser parameterParser,
                                     ParameterDeclarer<?> parameter, Supplier<Map<String, String>> containerParameterNames) {
    parameterParser.getValueProviderModelParser()
        .ifPresent(valueProviderModelParser -> {
          List<ActingParameterModel> actingParameterModels =
              getActingParameterModels(valueProviderModelParser, containerParameterNames.get());
          String vpName = valueProviderModelParser.getPartOrder().isPresent() ? group.getName() : parameterParser.getName();
          parameter.withValueProviderModel(createValueProviderModelFromParser(vpName,
                                                                              valueProviderModelParser,
                                                                              actingParameterModels));
          parameter
              .withModelProperty(new ValueProviderFactoryModelProperty(valueProviderModelParser.getValueProviderFactory(),
                                                                       actingParameterModels));
        });
  }

  private void declareFieldValueProviders(ParameterModelParser parameterParser,
                                          ParameterDeclarer<?> parameter, Supplier<Map<String, String>> containerParameterNames) {
    Map<String, ValueProviderFactoryModelProperty> fieldValueProviderFactoriesBySelector = new LinkedHashMap<>();
    List<FieldValueProviderModel> fieldValueProviderModels = new ArrayList<>();
    parameterParser.getFieldValueProvidersModelParser()
        .forEach(fieldValuesParser -> {
          List<ActingParameterModel> actingParameterModels =
              getActingParameterModels(fieldValuesParser, containerParameterNames.get());
          fieldValueProviderModels.addAll(createValueProviderModelsFromParser(parameterParser.getName(),
                                                                              fieldValuesParser,
                                                                              actingParameterModels));
          ValueProviderFactoryModelProperty factoryMP =
              new ValueProviderFactoryModelProperty(fieldValuesParser.getValueProviderFactory(), actingParameterModels);
          fieldValuesParser.getTargetSelectors()
              .forEach(targetSelector -> fieldValueProviderFactoriesBySelector.put(targetSelector, factoryMP));
        });

    if (!fieldValueProviderModels.isEmpty()) {
      parameter.withFieldValueProviderModels(fieldValueProviderModels);
    }
    if (!fieldValueProviderFactoriesBySelector.isEmpty()) {
      parameter.withModelProperty(new FieldsValueProviderFactoryModelProperty(fieldValueProviderFactoriesBySelector));
    }
  }

  private Map<String, String> getContainerParameterNames(List<ParameterGroupModelParser> groupParsers) {
    return groupParsers.stream()
        .flatMap(gp -> gp.getParameterParsers().stream())
        .collect(toMap(IntrospectionUtils::getImplementingName, ParameterModelParser::getName));
  }

  private ValueProviderModel createValueProviderModelFromParser(String valueProviderName,
                                                                ValueProviderModelParser valueProviderModelParser,
                                                                List<ActingParameterModel> actingParameterModels) {
    return new ValueProviderModel(actingParameterModels,
                                  valueProviderModelParser.requiresConfiguration(),
                                  valueProviderModelParser.requiresConnection(),
                                  valueProviderModelParser.isOpen(),
                                  valueProviderModelParser.getPartOrder().orElse(1),
                                  valueProviderName,
                                  valueProviderModelParser.getId(),
                                  valueProviderModelParser.getAdditionalModelProperties().toArray(new ModelProperty[0]));
  }

  private List<FieldValueProviderModel> createValueProviderModelsFromParser(String valueProviderName,
                                                                            FieldValueProviderModelParser valueProviderModelParser,
                                                                            List<ActingParameterModel> actingParameterModels) {
    List<FieldValueProviderModel> fieldValueProviderModels = new ArrayList<>();
    int partOrder = 1;
    valueProviderName = getFieldValueProviderName(valueProviderName,
                                                  valueProviderModelParser.getTargetSelectors().toArray(new String[0]));
    for (String targetSelector : valueProviderModelParser.getTargetSelectors()) {
      fieldValueProviderModels.add(new FieldValueProviderModel(actingParameterModels,
                                                               valueProviderModelParser.requiresConfiguration(),
                                                               valueProviderModelParser.requiresConnection(),
                                                               valueProviderModelParser.isOpen(),
                                                               partOrder++,
                                                               valueProviderName,
                                                               valueProviderModelParser.getId(),
                                                               targetSelector));

    }

    return fieldValueProviderModels;
  }

  private List<ActingParameterModel> getActingParameterModels(ValueProviderModelParser valueProviderModelParser,
                                                              Map<String, String> containerParameterNames) {
    return valueProviderModelParser.getActingParameterParsers().stream()
        .map(parameterParser -> getActingParameterModel(parameterParser, containerParameterNames))
        .toList();
  }

  private ActingParameterModel getActingParameterModel(ActingParameterModelParser actingParameterModelParser,
                                                       Map<String, String> containerParameterNames) {
    if (actingParameterModelParser.getExtractionExpression().isPresent()) {
      return new ImmutableActingParameterModel(actingParameterModelParser.getParameterName(),
                                               actingParameterModelParser.isRequired(),
                                               actingParameterModelParser.getExtractionExpression().get());

    } else {
      String aliasedName = containerParameterNames.get(actingParameterModelParser.getParameterName());
      return new ImmutableActingParameterModel(aliasedName, actingParameterModelParser.isRequired(), aliasedName);
    }
  }

  private MetadataKeyPartModelProperty createMetadataKeyPartModelProperty(Pair<Integer, Boolean> metadataKeyPart) {
    return new MetadataKeyPartModelProperty(metadataKeyPart.getFirst(), metadataKeyPart.getSecond());
  }
}
